@@ -1,7 +1,7 @@
 /*****************************************************************************
  * test_dr.h
  * (c)2001-2002 VideoLAN
- * $Id: test_dr.h 175 2008-07-13 12:52:18Z sam $
+ * $Id: test_dr.h 88 2004-02-24 14:31:18Z sam $
  *
  * Authors: Arnaud de Bossoreille de Ribou <bozo@via.ecp.fr>
  *
@@ -78,8 +78,7 @@
   if(!i_err && (s_decoded.name != p_new_decoded->name))                 \
   {                                                                     \
     fprintf(stderr, "\nError: integer %s %llu -> %llu\n", #name,        \
-            (long long unsigned int)s_decoded.name,                     \
-            (long long unsigned int)p_new_decoded->name);               \
+            (uint64_t)s_decoded.name, (uint64_t)p_new_decoded->name);   \
     i_err = 1;                                                          \
   }
 
