@@ -89,6 +89,7 @@ int dvbpsi_AttachEIT(dvbpsi_decoder_t * p_psi_decoder, uint8_t i_table_id,
   p_subdec->pf_callback = &dvbpsi_GatherEITSections;
   p_subdec->p_cb_data = p_eit_decoder;
   p_subdec->i_id = (uint32_t)i_table_id << 16 | (uint32_t)i_extension;
+  p_subdec->pf_detach = dvbpsi_DetachEIT;
 
   /* Attach the subtable decoder to the demux */
   p_subdec->p_next = p_demux->p_first_subdec;
@@ -116,7 +117,7 @@ void dvbpsi_DetachEIT(dvbpsi_demux_t * p_demux, uint8_t i_table_id,
           uint16_t i_extension)
 {
   dvbpsi_demux_subdec_t* p_subdec;
-  dvbpsi_demux_subdec_t* p_next_subdec;
+  dvbpsi_demux_subdec_t** pp_prev_subdec;
   dvbpsi_eit_decoder_t* p_eit_decoder;
 
   unsigned int i;
@@ -132,8 +133,6 @@ void dvbpsi_DetachEIT(dvbpsi_demux_t * p_demux, uint8_t i_table_id,
     return;
   }
 
-  p_next_subdec = p_subdec->p_next;
-
   p_eit_decoder = (dvbpsi_eit_decoder_t*)p_subdec->p_cb_data;
 
   free(p_eit_decoder->p_building_eit);
@@ -141,12 +140,17 @@ void dvbpsi_DetachEIT(dvbpsi_demux_t * p_demux, uint8_t i_table_id,
   for(i = 0; i <= 255; i++)
   {
     if(p_eit_decoder->ap_sections[i])
-      free(p_eit_decoder->ap_sections[i]);
+      dvbpsi_DeletePSISections(p_eit_decoder->ap_sections[i]);
   }
 
   free(p_subdec->p_cb_data);
+
+  pp_prev_subdec = &p_demux->p_first_subdec;
+  while(*pp_prev_subdec != p_subdec)
+    pp_prev_subdec = &(*pp_prev_subdec)->p_next;
+
+  *pp_prev_subdec = p_subdec->p_next;
   free(p_subdec);
-  p_subdec = p_next_subdec;
 }
 
 
@@ -401,6 +405,7 @@ void dvbpsi_GatherEITSections(dvbpsi_decoder_t * p_psi_decoder,
                      p_section->p_payload_start[4],
                      p_section->p_payload_start[5]);
       p_eit_decoder->i_last_section_number = p_section->i_last_number;
+      p_eit_decoder->i_first_received_section_number = p_section->i_number;
     }
 
     /* Fill the section array */
@@ -414,13 +419,50 @@ void dvbpsi_GatherEITSections(dvbpsi_decoder_t * p_psi_decoder,
 
     /* Check if we have all the sections */
     b_complete = 0;
-    for(i = 0; i <= p_eit_decoder->i_last_section_number; i++)
-    {
-      if(!p_eit_decoder->ap_sections[i])
-        break;
 
-      if(i == p_eit_decoder->i_last_section_number)
-        b_complete = 1;
+    /* As there may be gaps in the section_number fields (see below), we
+     * have to wait until we have received a section_number twice or
+     * until we have a received a section_number which is
+     * first_received_section_number - 1;
+     * if the first_received_section_number is 0, it's enough to wait
+     * until the last_section_number has been received;
+     * this is the only way to be sure that a complete table has been
+     * sent! */
+    if((p_eit_decoder->i_first_received_section_number > 0 &&
+        (p_section->i_number ==
+           p_eit_decoder->i_first_received_section_number ||
+         p_section->i_number ==
+           p_eit_decoder->i_first_received_section_number - 1)) ||
+       (p_eit_decoder->i_first_received_section_number == 0 &&
+        p_section->i_number == p_eit_decoder->i_last_section_number))
+    {
+      for(i = 0; i <= p_eit_decoder->i_last_section_number; i++)
+      {
+        if(!p_eit_decoder->ap_sections[i])
+          break;
+
+        if(i == p_eit_decoder->i_last_section_number)
+        {
+          b_complete = 1;
+          break;
+        }
+
+        /* ETSI EN 300 468 V1.5.1 section 5.2.4 says that the EIT
+         * sections may be structured into a number of segments and
+         * that there may be a gap in the section_number between
+         * two segments (but not within a single segment); thus at
+         * the end of a segment (indicated by
+         * section_number == segment_last_section_number)
+         * we have to search for the beginning of the next segment) */
+        if(i == p_eit_decoder->ap_sections[i]->p_payload_start[4])
+        {
+          while(!p_eit_decoder->ap_sections[i + 1] &&
+                (i + 1 < p_eit_decoder->i_last_section_number))
+          {
+            i++;
+          }
+        }
+      }
     }
 
     if(b_complete)
@@ -431,9 +473,17 @@ void dvbpsi_GatherEITSections(dvbpsi_decoder_t * p_psi_decoder,
       /* Chain the sections */
       if(p_eit_decoder->i_last_section_number)
       {
-        for(i = 0; i <= p_eit_decoder->i_last_section_number - 1; i++)
-          p_eit_decoder->ap_sections[i]->p_next =
-                                        p_eit_decoder->ap_sections[i + 1];
+        dvbpsi_psi_section_t * p_prev_section;
+
+        p_prev_section = p_eit_decoder->ap_sections[0];
+        for(i = 1; i <= p_eit_decoder->i_last_section_number; i++)
+        {
+          if(p_eit_decoder->ap_sections[i] != NULL)
+          {
+            p_prev_section->p_next = p_eit_decoder->ap_sections[i];
+            p_prev_section = p_eit_decoder->ap_sections[i];
+          }
+        }
       }
       /* Decode the sections */
       dvbpsi_DecodeEITSections(p_eit_decoder->p_building_eit,
